# -*- coding: utf-8 -*-

"""
The `minor_version` argument was deprecated in 0.1.27 and should be removed in 0.2.0.
"""

import os
import warnings

import productmd


class VersionType(object):
    MAJOR = 0
    MINOR = 1
    FULL = 2


class ComposeSymlink(object):
    def __init__(self, compose_path):
        self.compose = productmd.compose.Compose(compose_path)
        # The path points to the compose/ subdirectory.
        self.path = os.path.dirname(self.compose.compose_path)

    def _link(self, name):
        """Create a symbolic link with given name at the same location as the
        compose. Existing file at the given path will be deleted atomically.
        """
        temp = os.path.join(self.path, "..", ".%s.tmp" % name)
        symlink = os.path.join(self.path, "..", name)

        os.symlink(os.path.basename(self.path), temp)
        os.rename(temp, symlink)

    def get_latest_link_name(self, minor_version=None, version_type=VersionType.MAJOR):
        """Get name of latest- symlink for current compose."""
        if minor_version is not None:
            _deprecated_arg("minor_version", "version_type")
        if minor_version:
            version_type = VersionType.MINOR

        formats = {
            VersionType.MAJOR: "{0.release.major_version}",
            VersionType.MINOR: "{0.release.major_version}.{0.release.minor_version}",
            VersionType.FULL: "{0.release.version}",
        }
        version = formats[version_type].format(self.compose.info)

        fmt = "latest-{0.release.short}-{1}"
        if self.compose.info.release.is_layered:
            # Add info about base product
            fmt += '-{0.base_product.short}-{0.base_product.version}'
            if self.compose.info.base_product.type != "ga":
                # Add base product type only if it's not ga
                fmt += "-{0.base_product.type}"
        return fmt.format(self.compose.info, version)

    def print_latest_link(self, minor_version=None, version_type=VersionType.MAJOR):
        """Print a latest- symlink with configured version pattern."""
        if minor_version is not None:
            _deprecated_arg("minor_version", "version_type")
        if minor_version:
            version_type = VersionType.MINOR
        print(self.get_latest_link_name(version_type=version_type))

    def create_latest_link(self, minor_version=None, version_type=VersionType.MAJOR):
        """Create a latest- symlink with configured version pattern."""
        if minor_version is not None:
            _deprecated_arg("minor_version", "version_type")
        if minor_version:
            version_type = VersionType.MINOR
        self._link(self.get_latest_link_name(version_type=version_type))


def print_latest_link(compose_path, minor_version=None, version_type=VersionType.MAJOR):
    """Convenience function for easy printing of latest symlink."""
    if minor_version is not None:
        _deprecated_arg("minor_version", "version_type")
    if minor_version:
        version_type = VersionType.MINOR
    ComposeSymlink(compose_path).print_latest_link(version_type=version_type)


def create_latest_link(compose_path, minor_version=None, version_type=VersionType.MAJOR):
    """Convenience function for easy creation of latest symlink."""
    if minor_version is not None:
        _deprecated_arg("minor_version", "version_type")
    if minor_version:
        version_type = VersionType.MINOR
    ComposeSymlink(compose_path).create_latest_link(version_type=version_type)


def _deprecated_arg(arg, replace):
    warnings.warn(
        "%r argument is deprecated, use %r instead" % (arg, replace), DeprecationWarning
    )
